
///////////////////////////////////////////////////////////////////////////////////////////////
//
//
// Context class implementation
//
//
///////////////////////////////////////////////////////////////////////////////////////////////
// Stephane Rodriguez
//


#include "stdafx.h"
#include "HostSite.h"
#include <atlbase.h>
#include <comutil.h>
#include "Context.h"
#include "xmlwriter.h"



void AfxBSTR2CString(BSTR bstrstring, CString &szOutput);

CContext::CContext()
{
	init();
}
CContext::~CContext()
{

	if (m_szName)
	{
		char szContextName[2048];
		sprintf(szContextName,"---%s context stopped---\n", m_szName);
		OutputDebugString (szContextName);
	}

	delete [] m_szName;
}
void CContext::setName(LPTSTR name)
{
	m_szName = new TCHAR[ _tcslen(name)+1 ];
	if (m_szName)
		_tcscpy(m_szName, name);
}
void CContext::setHostSite(CHostSite *pHost)
{
	m_cpHostSite = pHost;
}
CHostSite *CContext::getHostSite()
{
	return m_cpHostSite;
}

void CContext::setUserData(LPVOID pData)
{
	m_cpUserData = pData;
}

LPVOID CContext::getUserData()
{
	return m_cpUserData;
}


void CContext::setParent(CContext *pParent)
{
	m_cpParentContext = pParent;
}

CContext *CContext::getParent()
{
	return m_cpParentContext;
}


void CContext::init()
{
	setUserData(NULL);
	setHostSite(NULL);
	setParent(NULL);
	m_szName = NULL;
}

void CContext::apply(/*in*/CStringArray &ar)
{
	USES_CONVERSION; // helper for A2W implicit conversion

	if (m_szName)
	{
		char szContextName[2048];
		sprintf(szContextName,"---%s context running---\n", m_szName);
		OutputDebugString (szContextName);
	}

	CContext *pParent = getParent();
	if (!pParent)
		return;

	IDispatch *pCurInterface = (IDispatch *) pParent->getUserData();
	if (!pCurInterface)
		return;

	if (ar.GetSize()==0) // no attribute passed at all
	{
		// TO DO
		//
		// might be of interest to try a query interface, for instance in this case :
		// <IApplication>
		//   <IDocuments>
		return;
	}

	if (_tcsicmp(ar.GetAt(0),IPROPERTY)==0)	// case atts[0]=property
	{
		VARIANT FAR VarResult;
		CDoItInvokeHelper h;
		
		HRESULT hr = h.InvokeHelper_PropertyGet(pCurInterface, 
												A2W(ar.GetAt(1)), //L"Documents", 
												VarResult);

		if (SUCCEEDED(hr))
		{
			if (VarResult.vt==VT_DISPATCH)
				setUserData( VarResult.pdispVal );
			else
			{
				int c = 0;
			}
		}

	} // end // case property="<propertyname>"
	else if (ar.GetSize()>2 && ar.GetAt(1).Compare("*") ) // discovery method
	{
		CString szFilename;
		if (_tcsicmp(ar.GetAt(2),IOUTPUTXMLFILENAME)==0) // second attribute = "output" ?
			szFilename = ar.GetAt(3); // get fully qualified filepath of output

		discoverInterface(pCurInterface,szFilename);
	}
	else
	{
		// other case, atts[0]= any method name
		//

		CString szMethod = ar.GetAt(0);

		// atts[0] = "open" (cmd)
		// atts[1] = "docname" (params = "xx,yyy,zzzzz")
		DISPPARAMS dispparams;
		dispparams.rgvarg = NULL;
		dispparams.cArgs = 0;
		dispparams.cNamedArgs = 0;
		INVOKEKIND invKind = INVOKE_FUNC; // default invoke method

		buildDispParams(pCurInterface,ar.GetAt(0),ar.GetAt(1), &dispparams, &invKind);

		VARIANT FAR VarResult;
		CDoItInvokeHelper h;
		
		HRESULT hr = S_OK;
		
		if (invKind == INVOKE_PROPERTYPUT)
		{
			CString szProperty = ar.GetAt(0);
			hr = h.InvokeHelper_PropertyPutDISPPARAMS(pCurInterface, 
														A2W( szProperty.GetBuffer(0) ), //L"Open", 
														dispparams,
														VarResult);
		}
		else if (invKind == INVOKE_PROPERTYGET)
		{
			CString szProperty = ar.GetAt(0);
			hr = h.InvokeHelper_PropertyGet(pCurInterface, 
											A2W( szProperty.GetBuffer(0) ), //L"Open", 
											VarResult);
		}
		else
		{
			hr = h.InvokeHelper_Method_ParamDISPPARAMS(pCurInterface, 
														A2W( ar.GetAt(0) ), //L"Open", 
														dispparams,
														VarResult);
		}

		if (SUCCEEDED(hr))
		{
			if (VarResult.vt==VT_DISPATCH)
				setUserData( VarResult.pdispVal );
			else if (VarResult.vt==VT_EMPTY)
				setUserData( NULL );
			else
			{

				int c = 0;

				// TODO : what do we do with a non-IDispatch result ?
			}
		}


		// free all BSTR strings
		if (dispparams.rgvarg)
		{
			int nbParams = dispparams.cArgs;
			for (int p=0;p<nbParams;p++)
			{
				if (dispparams.rgvarg[p].vt == VT_BSTR)
					::SysFreeString(dispparams.rgvarg[p].bstrVal);
			}
		}

		// Free internal object of DISPPARAMS
		delete [] dispparams.rgvarg;

	}
}



BOOL looksLikeFloat(CString &st)
{
	if (st.IsEmpty())
		return FALSE;

	register char c;
	char *s=st.GetBuffer(0);
	while (*s!='\0')
	{
		c = *s;
		s++;
		if ((c<'0' || c>'9') && !((c=='.') || (c=='e') || (c=='-') || (c=='+')))
			return FALSE;
	}

	return TRUE;
}

BOOL looksLikeNumber(CString &st)
{
	if (st.IsEmpty())
		return FALSE;

	register char c;
	char *s=st.GetBuffer(0);
	while (*s!='\0')
	{
		c = *s;
		s++;
		if ((c<'0' || c>'9') && !((c=='-') || (c=='+')))
			return FALSE;
	}

	return TRUE;
}


BOOL looksLikeBoolean(CString &s)
{
	return ( _tcsicmp(s,"true")==0 || _tcsicmp(s,"false")==0 ) ? TRUE : FALSE;
}



void CContext::buildDispParams(/*in*/IDispatch *pDispatch, 
							   /*in*/LPTSTR cmd, /*in*/LPTSTR params, 
							   /*out*/DISPPARAMS *pDispParams,
							   /*out*/INVOKEKIND *pinvKind)
{
	if (!pDispatch || !cmd || !pDispParams)
		return;

	// tokenize params using the ',' separator
	CString sParams = params;

	CStringArray sa;
	long iStart = 0;
	CString st = sParams.Tokenize(",",iStart);
	while ( !st.IsEmpty() )
	{
		sa.Add(st);
		st = sParams.Tokenize(",",iStart);
	}
	

	int nbParams = (int) sa.GetSize();
/*	if (nbParams==0)
	{
		pDispParams->rgvarg = NULL;
		pDispParams->cArgs = 0;
		pDispParams->cNamedArgs = 0;
		return; // no param, that's the simplest thing to manage!
	}*/

	CString sCmd = cmd;

	// now try to find whether we are dealing with numbers, floats, booleans, or strings
	//

	// a good idea here to retrieve all expected types for the known call and try to match them
	//
	while(1) // just to be able to do break;
	{
		unsigned int FAR nbtypeinfoInterfaces = 0;
		if (SUCCEEDED(pDispatch->GetTypeInfoCount(&nbtypeinfoInterfaces)) && nbtypeinfoInterfaces>0)
		{
			BOOL bMethodExists = FALSE;
			unsigned short nActualFunc = -1;

			ITypeInfo FAR *pInfo = NULL;
			pDispatch->GetTypeInfo( 0/* unsigned int  iTInfo*/,         
									0 /*LCID  lcid*/,                   
									&pInfo);
			if (!pInfo)
				break;

			TYPEATTR FAR *typeattr=NULL;
			pInfo->GetTypeAttr( &typeattr );
			unsigned short cFuncs = typeattr->cFuncs;
			pInfo->ReleaseTypeAttr( typeattr );

			if (cFuncs<=0)
			{
				pInfo->Release();
				break;
			}

			
			unsigned short nFuncs=0;
			while (nFuncs<cFuncs && !bMethodExists)
			{
				FUNCDESC FAR *funcdesc=NULL;
				if (SUCCEEDED(pInfo->GetFuncDesc(  (unsigned int) nFuncs, &funcdesc )))
				{
					MEMBERID idMember;
					idMember = funcdesc->memid;
					BSTR bstrMethodName;
					pInfo->GetDocumentation(idMember, &bstrMethodName, NULL, NULL, NULL);
					//
					CString szMethodName;
					AfxBSTR2CString(bstrMethodName, szMethodName);
					bMethodExists = szMethodName.CompareNoCase(sCmd);
					if (bMethodExists)
						nActualFunc = nFuncs;
					//
					::SysFreeString(bstrMethodName);
					pInfo->ReleaseFuncDesc(funcdesc);
				}

				nFuncs++;
			}

			// ok now try to match IDL params with XML params
			//
			if (bMethodExists) // method was found in IDL type library
			{
				// help description of FUNCDESC structure provided by ITypeInfo::GetFuncDec() :
				//
				//	IDispatch::GetTypeInfo(&ITypeInfo)
				//	FUNCDESC.cParams = nb of params
				//	FUNCDESC.cParamsOpt = nb of optional params
				//	FUNCDESC.lprgelemdescParam = ELEMDESC[]
				//	ELEMDESC.tdesc = TYPEDESC
				//	TYPEDESC.vt = VARTYPE (VT_I4, ....)
				//

				FUNCDESC FAR *funcdesc=NULL;
				if (SUCCEEDED(pInfo->GetFuncDesc(  nActualFunc, &funcdesc )))
				{
					*pinvKind = funcdesc->invkind;

					if (nbParams > 0)
					{
						VARIANT *tempStruct= new VARIANT[nbParams]; // Remember to free VARIANT[]

						//
						int n=0;
						int nIDL=0;
						while (n<nbParams && nIDL<funcdesc->cParams)
						{
							// get expected IDL param type
							VARTYPE vt = funcdesc->lprgelemdescParam[nIDL].tdesc.vt;

							CString s = sa.GetAt(n);

							if ( vt==VT_I4)
							{
								tempStruct[nbParams-1 - n].vt = vt;
								tempStruct[nbParams-1 - n].lVal = atol( (LPCTSTR)s );
							}
							else if ( vt==VT_INT )
							{
								tempStruct[nbParams-1 - n].vt = vt;
								tempStruct[nbParams-1 - n].intVal = atol( (LPCTSTR)s );
							}
							else if ( vt==VT_R8 )
							{
								tempStruct[nbParams-1 - n].vt = VT_R8;
								tempStruct[nbParams-1 - n].dblVal = atof( (LPCTSTR)s );
							}
							else if ( vt==VT_BOOL )
							{
								tempStruct[nbParams-1 - n].vt = VT_BOOL;
								tempStruct[nbParams-1 - n].boolVal = _tcsicmp(s,"true")==0 ? VARIANT_TRUE : VARIANT_FALSE;
							}
							else // default as string
							{
								BSTR bstrParam = s.AllocSysString(); // Remember to free BSTR string
								tempStruct[nbParams-1 - n].vt = VT_BSTR;
								tempStruct[nbParams-1 - n].bstrVal = bstrParam;
							}

							//
							n++;
							nIDL++;
						} // end while

						pDispParams->rgvarg = tempStruct;
						pDispParams->cArgs = nbParams;
						pDispParams->cNamedArgs = 0;

					} // end if (nbParams > 0)

					//
					pInfo->ReleaseFuncDesc(funcdesc);

				} // end if (SUCCEEDED(pInfo->GetFuncDesc(  nActualFunc, &funcdesc )))

				pInfo->Release();
				return; // we can end now, all params built and ready for invoke

			} // end if (bMethodExists

			if (pInfo)
				pInfo->Release();

		} // end pDispatch->GetTypeInfoCount(&nbtypeinfoInterfaces))

		break;

	}  // end while

	if (nbParams == 0) return;

	// no run-time information ==> blind method call
	//
	VARIANT *tempStruct= new VARIANT[nbParams]; // Remember to free VARIANT[]

	for (int i=0;i<nbParams;i++)
	{
		CString s = sa.GetAt(i);

		if ( looksLikeFloat(s) )
		{
			tempStruct[nbParams-1 - i].vt = VT_R8;
			tempStruct[nbParams-1 - i].dblVal = atof( (LPCTSTR)s );
		}
		else if ( looksLikeNumber(s) )
		{
			tempStruct[nbParams-1 - i].vt = VT_I4;
			tempStruct[nbParams-1 - i].lVal = atol( (LPCTSTR)s );
		}
		else if ( looksLikeBoolean(s) )
		{
			tempStruct[nbParams-1 - i].vt = VT_BOOL;
			tempStruct[nbParams-1 - i].boolVal = _tcsicmp(s,"true")==0 ? VARIANT_TRUE : VARIANT_FALSE;
		}
		else // default as string
		{
			BSTR bstrParam = s.AllocSysString(); // Remember to free BSTR string
			tempStruct[nbParams-1 - i].vt = VT_BSTR;
			tempStruct[nbParams-1 - i].bstrVal = bstrParam;
		}
	}

	pDispParams->rgvarg = tempStruct;
	pDispParams->cArgs = nbParams;
	pDispParams->cNamedArgs = 0;

}


void CContext::discoverInterface(/*in*/IDispatch *pDispatch, /*in*/CString &szOutputFilename)
{
	if (!pDispatch)	return;

	unsigned int FAR nbtypeinfoInterfaces = 0;
	if (SUCCEEDED(pDispatch->GetTypeInfoCount(&nbtypeinfoInterfaces)) && nbtypeinfoInterfaces>0)
	{

		ITypeInfo FAR *pInfo = NULL;
		pDispatch->GetTypeInfo( 0/* unsigned int  iTInfo*/,         
								0 /*LCID  lcid*/,                   
								&pInfo);
		if (!pInfo)
			return;

		XmlWriter w;
		XmlElement winterface( m_szName );

		if (szOutputFilename.GetLength()>0)
		{
			w.Open( szOutputFilename );
			winterface.Write(w,1);
		}


		TYPEATTR FAR *typeattr=NULL;
		pInfo->GetTypeAttr( &typeattr );
		unsigned short cFuncs = typeattr->cFuncs;
		pInfo->ReleaseTypeAttr( typeattr );

		if (cFuncs<=0)
		{

			if (szOutputFilename.GetLength()>0)
			{
				winterface.WriteClosingTag(w,-1);
				w.Close();
			}

			pInfo->Release();
			return;
		}

		
		// list all properties and methods
		unsigned short nFuncs=0;
		unsigned short nActualFunc = -1;
		while (nFuncs<cFuncs)
		{
			FUNCDESC FAR *funcdesc=NULL;
			if (SUCCEEDED(pInfo->GetFuncDesc(  (unsigned int) nFuncs, &funcdesc )))
			{
				MEMBERID idMember;
				idMember = funcdesc->memid;
				BSTR bstrMethodName;
				pInfo->GetDocumentation(idMember, &bstrMethodName, NULL, NULL, NULL);
				//
				CString szMethodName;
				AfxBSTR2CString(bstrMethodName, szMethodName);
				::SysFreeString(bstrMethodName);

				BSTR FAR* rgbstrNames = NULL;
				unsigned int cNames = 0;
				rgbstrNames = new BSTR[funcdesc->cParams+1];
				pInfo->GetNames(idMember, rgbstrNames, funcdesc->cParams+1, &cNames);


				// help description of FUNCDESC structure provided by ITypeInfo::GetFuncDec() :
				//
				//	IDispatch::GetTypeInfo(&ITypeInfo)
				//	FUNCDESC.cParams = nb of params
				//	FUNCDESC.cParamsOpt = nb of optional params
				//	FUNCDESC.lprgelemdescParam = ELEMDESC[]
				//	ELEMDESC.tdesc = TYPEDESC
				//	TYPEDESC.vt = VARTYPE (VT_I4, ....)
				//

				CString sIN, sOUT;

				short n=0;
				short nbparams = funcdesc->cParams;
				short nboptionalparams = funcdesc->cParamsOpt;

				while (n < nbparams)
				{
					// get expected IDL param type
					VARTYPE vt = funcdesc->lprgelemdescParam[n].tdesc.vt;
					short nParamFlag = funcdesc->lprgelemdescParam[n].paramdesc.wParamFlags;
					// PARAMFLAG definition (from MSDN) :
					//#define PARAMFLAG_NONE			0
					//#define PARAMFLAG_FIN				0x1
					//#define PARAMFLAG_FOUT			0x2
					//#define PARAMFLAG_FLCID			0x4
					//#define PARAMFLAG_FRETVAL			0x8
					//#define PARAMFLAG_FOPT			0x10
					//#define PARAMFLAG_FHASDEFAULT		0x20

					int nAddPtr = 0;

					TYPEDESC *td = &(funcdesc->lprgelemdescParam[n].tdesc);
					while (vt==VT_PTR && td && td->lptdesc)
					{
						vt = td->lptdesc->vt;

						td = td->lptdesc;

						nAddPtr++;
					}

					BOOL bIsOut = (nParamFlag & PARAMFLAG_FOUT) != 0;
					BOOL bIsRetval = (nParamFlag & PARAMFLAG_FRETVAL) != 0;



					if (vt==VT_USERDEFINED)
					{
						if ( (nParamFlag & PARAMFLAG_FOUT)==0 || (nParamFlag & PARAMFLAG_FRETVAL)==0)
							buildStringFromUserdefinedtype(pInfo, td, 
													isOptionalParam(n,nbparams,nboptionalparams), 
													nParamFlag, 
													nAddPtr, 
													(n+1<short(cNames)) ? rgbstrNames[n+1] : NULL,
													sIN);
						else
							buildStringFromUserdefinedtype(pInfo, td, 
													FALSE, 
													nParamFlag, 
													nAddPtr, 
													(n+1<short(cNames)) ? rgbstrNames[n+1] : NULL,
													sOUT);
					}


					if ( (nParamFlag & PARAMFLAG_FOUT)==0 || (nParamFlag & PARAMFLAG_FRETVAL)==0)
						buildStringFromVartype(vt, 
												isOptionalParam(n,nbparams,nboptionalparams), 
												nParamFlag, 
												nAddPtr, 
												(n+1<short(cNames)) ? rgbstrNames[n+1] : NULL,
												sIN);
					else
						buildStringFromVartype(vt, 
												FALSE, 
												nParamFlag, 
												nAddPtr, 
												(n+1<short(cNames)) ? rgbstrNames[n+1] : NULL,
												sOUT);

					n++;
				} // end while


				// return type of the function
				if (funcdesc->cScodes>0)
				{
					VARTYPE vt = funcdesc->elemdescFunc.tdesc.vt;
					short nParamFlag = funcdesc->elemdescFunc.paramdesc.wParamFlags;

					int nAddPtr = 0;

					TYPEDESC *td = &(funcdesc->elemdescFunc.tdesc);
					while (vt==VT_PTR && td && td->lptdesc)
					{
						vt = td->lptdesc->vt;

						td = td->lptdesc;

						nAddPtr++;
					}

					if (vt==VT_USERDEFINED)
						buildStringFromUserdefinedtype(pInfo, td, FALSE, nParamFlag, nAddPtr, NULL,	sOUT);
					else
						buildStringFromVartype(vt, FALSE, nParamFlag, nAddPtr, NULL, sOUT);
				}

				pInfo->ReleaseFuncDesc(funcdesc);

				if (szOutputFilename.GetLength()>0)
				{
					XmlElement wmethod( szMethodName.GetData() );

					wmethod.AddAttrib( CString(IOUTPUTXMLPARAMS), sIN );
					wmethod.AddAttrib( CString(IOUTPUTXMLRETURNVALUE), sOUT );
					wmethod.WriteEmpty(w,1);
				}

				// release all BSTR stuff
				for (unsigned short m=0;m<cNames;m++)
					::SysFreeString( rgbstrNames[m] );

				delete [] rgbstrNames;
				

			} // end struct FUNCDESC properly retrieved

			nFuncs++;

		} // for each func

		if (pInfo)
			pInfo->Release();


		if (szOutputFilename.GetLength()>0)
		{
			winterface.WriteClosingTag(w,-1);
			w.Close();
		}



	} // end pDispatch->GetTypeInfoCount(&nbtypeinfoInterfaces))


}


void CContext::buildStringFromVartype(VARTYPE vt, BOOL bIsOptional, short nParamFlag, int nAddPtr, BSTR varname, CString &s)
{
	CString t;

	switch (vt)
	{
		case VT_EMPTY : break;
		case VT_NULL : break;
		case VT_UI1 : t = "unsigned char"; break;
		case VT_UI2 : t = "unsigned short"; break;
		case VT_UI4 : t = "unsigned short"; break; // !
		case VT_UI8 : t = "unsigned 64-bit long"; break;
		case VT_I1 : t = "char"; break;
		case VT_I2 : t = "short"; break;
		case VT_I4 : t = "long"; break;
		case VT_I8 : t = "64-bit long"; break;
		case VT_R4 : t = "float"; break;
		case VT_R8 : t = "double"; break;
		case VT_BOOL : t = "VARIANT_BOOL"; break;
		case VT_ERROR : t = "SCODE"; break;
		case VT_CY : t = "CY"; break;
		case VT_DATE : t = "DATE"; break;
		case VT_BSTR : t = "BSTR"; break;
		case VT_UNKNOWN : t = "IUnknown"; break;
		case VT_DISPATCH : t = "IDispatch"; break;
		case VT_ARRAY : t = "SAFEARRAY*"; break;
		case VT_VARIANT : t = "VARIANT"; break;
		case VT_BYREF|VT_UI1 : t = "unsigned char*"; break;
		case VT_BYREF|VT_UI2 : t = "unsigned short*"; break;
		case VT_BYREF|VT_UI4 : t = "unsigned short*"; break;
		case VT_BYREF|VT_UI8 : t = "unsigned 64-bit long*"; break;
        case VT_BYREF|VT_I1 : t = "char*"; break;
		case VT_BYREF|VT_I2 : t = "short*"; break;
        case VT_BYREF|VT_I4 : t = "long*"; break;
		case VT_BYREF|VT_I8 : t = "64-bit long*"; break;
        case VT_BYREF|VT_R4 : t = "float*"; break;
        case VT_BYREF|VT_R8 : t = "double*"; break;
        case VT_BYREF|VT_BOOL : t = "VARIANT_BOOL*"; break;
        case VT_BYREF|VT_ERROR : t = "SCODE*"; break;
        case VT_BYREF|VT_CY : t = "CY*"; break;
        case VT_BYREF|VT_DATE : t = "DATE*"; break;
        case VT_BYREF|VT_BSTR : t = "BSTR*"; break;
        case VT_BYREF|VT_UNKNOWN : t = "IUnknown*"; break;
        case VT_BYREF|VT_DISPATCH : t = "IDispatch*"; break;
        case VT_BYREF|VT_VARIANT : t = "VARIANT*"; break;
	}

	if (t.IsEmpty())
		return;

	if (!s.IsEmpty())
		s += ","; // add separator

	if (bIsOptional)
		s += "[optional]";

	if (nParamFlag & PARAMFLAG_FIN)
		s += "[in]";
	else if ( (nParamFlag & PARAMFLAG_FRETVAL)==0 && (nParamFlag & PARAMFLAG_FOUT)==1 )
		s += "[out]";

	if (nParamFlag & PARAMFLAG_FLCID)
		s += "[LCID]";

	s += t;

	if (nAddPtr>0)
	{
		int c = nAddPtr;
		while (c-->0)
			s += "*";
	}

	if (varname) // BSTR
	{
		CString sVarname;
		AfxBSTR2CString(varname, sVarname);
		s += " ";
		s += sVarname;
		::SysFreeString(varname);
	}

	if (nParamFlag & PARAMFLAG_FHASDEFAULT)
		s += " = xxx"; // default value

}


void CContext::buildStringFromUserdefinedtype(ITypeInfo *pInfo, TYPEDESC *td, BOOL bIsOptional, short nParamFlag, int nAddPtr, BSTR varname, CString &s)
{
	if (!pInfo || !td) return;

	ITypeInfo FAR *pInfoUserDefined = NULL;
	if (FAILED(pInfo->GetRefTypeInfo( td->hreftype, &pInfoUserDefined)))
		return;

	if (!pInfoUserDefined) return;

	BSTR bstrMethodName;
	MEMBERID idMember = MEMBERID_NIL;
	pInfoUserDefined->GetDocumentation(idMember, &bstrMethodName, NULL, NULL, NULL);
	//
	CString t;
	AfxBSTR2CString(bstrMethodName, t);
	::SysFreeString(bstrMethodName);

	pInfoUserDefined->Release();

	if (t.IsEmpty())
		return;

	if (!s.IsEmpty())
		s += ","; // add separator

	if (bIsOptional)
		s += "[optional]";

	if (nParamFlag & PARAMFLAG_FIN)
		s += "[in]";
	else if ( (nParamFlag & PARAMFLAG_FRETVAL)==0 && (nParamFlag & PARAMFLAG_FOUT)==1 )
		s += "[out]";


	if (nParamFlag & PARAMFLAG_FLCID)
		s += "[LCID]";

	s += t;

	if (nAddPtr>0)
	{
		int c = nAddPtr;
		while (c-->0)
			s += "*";
	}

	if (varname) // BSTR
	{
		CString sVarname;
		AfxBSTR2CString(varname, sVarname);
		s += " ";
		s += sVarname;
		::SysFreeString(varname);
	}

	if (nParamFlag & PARAMFLAG_FHASDEFAULT)
		s += " = xxx"; // default value


}

BOOL CContext::isOptionalParam(int i, short nbparams, short nboptionalparams)
{
	if (nboptionalparams==0)
		return FALSE;

	return (i >= (nbparams-nboptionalparams) ) ? TRUE : FALSE;
}

void AfxBSTR2CString(BSTR bstrstring, CString &szOutput)
{
	szOutput.Empty();

	_bstr_t bstrTag;
	bstrTag = bstrstring;

	UINT uLen = bstrTag.length();
	TCHAR *szTemp= new TCHAR[uLen+1];
	memset (szTemp, 0, uLen+1);

	LPOLESTR wszTag = bstrTag;

	int iBytes = ::WideCharToMultiByte(CP_ACP, 0,
			wszTag, uLen,
			szTemp, uLen, NULL, NULL);

	szOutput = szTemp;

	delete [] szTemp;
}





CServerContext::CServerContext() 
{
	m_pServerObj = NULL;
}
CServerContext::~CServerContext() 
{
	// destroy COM instance

	if (!m_pServerObj)
		return;

/* is there any method we should call against the COM server
	before we quit it ?

	If so, then implement it here :

	// prepare to quit the process
	//
	VARIANT FAR VarResult;
	CDoItInvokeHelper h;
	
	h.InvokeHelper_Method_NoParam(m_pServerObj, 
								  L"Quit", 
								  VarResult);

*/

	// good bye
	//
	m_pServerObj->Release();

	m_pServerObj = NULL; // sanity check

}
	
void CServerContext::init()  // do nothing here, just wait apply() to instantiate the COM server
{
}

void CServerContext::apply(/*in*/CStringArray &ar) 
{


	if (m_pServerObj!=NULL)
		return; // already started

	CString szclsid;

	// find server= attribute
	int i = 0;
	BOOL bStop = FALSE;
	long nSize = ar.GetSize() / 2; // attribute are stored by pairs
	while (i<nSize && !bStop)
	{
		if ( ar.GetAt(2*i).CompareNoCase(ISERVERCLSID) )
		{
			szclsid = ar.GetAt(2*i+1); // get value
			bStop = TRUE;
		}

		i++; 
	}

	if (szclsid.GetLength()==0)
	{
		char szError[2048];
		sprintf(szError,"You must pass <server registry=\"a clsid or a progid\">...</server>\r\nFor instance, <server registry=\"{5D0CE852-D909-11CF-91FC-00A0C903976F}\">...</server>   (a CLSID)\r\nOr        <server registry=\"COMServer.srvbctrl\">...</server>   (a progID)\r\n");
		OutputDebugString (szError);
		return;
	}

	// add EOL
	szclsid += "A";
	szclsid.SetAt( szclsid.GetLength()-1, 0); // a bit tricky but anyway...

	// convert 8-bit to wide-char
	long nSizeCLSID = szclsid.GetLength()+1;
	LPOLESTR wszclsid = (LPOLESTR) _alloca( 2*nSizeCLSID); // allocate on stack, automatically freed
	wszclsid[0] = '\0';
	::MultiByteToWideChar(CP_ACP, 0, szclsid, nSizeCLSID, wszclsid, nSizeCLSID);


	HRESULT hr;
	CLSID comserver;

	hr = CLSIDFromProgID(wszclsid,&comserver); // L"COMServer.ObjectTwo"
	if (FAILED(hr)) // param is either not a progID, or simply not correctly registered
	{
		// let's try as a CLSID
		hr = CLSIDFromString(wszclsid,&comserver); // L"{5D0CE852-D909-11CF-91FC-00A0C903976F}"
		if (FAILED(hr))
		{

			char szContextName[2048];
			sprintf(szContextName,"---%s context cannot run---\nEither XML GUID or COM registration %s is wrong\n", m_szName, szclsid);
			OutputDebugString (szContextName);

			return;
		}
	}

	
	if (m_szName)
	{
		char szContextName[2048];
		sprintf(szContextName,"---%s %s context running---\n", szclsid, m_szName);
		OutputDebugString (szContextName);
	}



    // Create an instance
	//
	if (FAILED(hr = CoCreateInstance(	comserver, 
										NULL, 
										CLSCTX_SERVER /*dll or exe*/, 
										IID_IDispatch, 
										(LPVOID*)&m_pServerObj ) ))
	{
		char szError[MAX_PATH];
		sprintf(szError,"CoCreateInstance failed");

		::MessageBox(NULL,szError,"Fatal error",MB_OK);
		strcat(szError,"\n");
		OutputDebugString(szError);
		return;
	}

	setUserData( m_pServerObj ); // store COM server interface


}







