
///////////////////////////////////////////////////////////////////////////////////////////////
//
//
// Context class declaration
//
//
///////////////////////////////////////////////////////////////////////////////////////////////
// Stephane Rodriguez
//

#ifndef __CCONTEXT__
#define __CCONTEXT__

class CHostSite; // forward declaration

#define ISERVERCTX			"server"
#define ISERVERCLSID		"registry"
#define IPROPERTY			"property"
#define IOUTPUTXMLPARAMS	"params"
#define IOUTPUTXMLRETURNVALUE "return"
#define IOUTPUTXMLFILENAME	"output"

/*
to define attachment to a com server, use XML like this :

  <server registry="{5D0CE852-D909-11CF-91FC-00A0C903976F}"> <!-- passing the GUID of the COM server -->
   ...
  </server>

  or 

  <server registry="COMServer.comsrvbctrl"> <!-- passing the progID of the COM server -->
   ...
  </server>
*/


/* properties, to be used like this :

   <xxx property="propertyname">
     ...
   </xxx>


*/


/* discovery method 

   <xxx yyy="*" output="c:\temp.xml">
     ...
   </xxx>

  the output attribute is optional. When present, records discovery results to the named file.

*/



class CContext
{
	// Members
	//
protected:
	char*		m_szName;
	CHostSite*	m_cpHostSite; // copy ptr
	LPVOID		m_cpUserData; // copy ptr
	CContext*	m_cpParentContext;

	// Construction
	//
public:
	CContext();
	virtual ~CContext();

	// Accessors
	//
public:
	void		setName(LPTSTR name);
	void		setHostSite(CHostSite *pHost);
	CHostSite	*getHostSite();
	void		setUserData(LPVOID pData);
	LPVOID		getUserData();
	void		setParent(CContext *pParent);
	CContext*	getParent();

	// Methods
	//
public:
	virtual void init();
	virtual void apply(/*in*/CStringArray &ar); // override this method




	// Helpers
	//
protected:
	void buildDispParams(/*in*/IDispatch *pDispatch, 
						/*in*/LPTSTR cmd, /*in*/LPTSTR params, 
						/*out*/DISPPARAMS *pDispParams,
						/*out*/INVOKEKIND *pinvKind);
	void discoverInterface(/*in*/IDispatch *pDispatch, /*in*/CString &szOutputFilename);
	void buildStringFromVartype(VARTYPE vt, BOOL bIsOptional, short nParamFlag, int nAddPtr, BSTR varname, CString &s); // used by discoverInterface
	void buildStringFromUserdefinedtype(ITypeInfo *pInfo, TYPEDESC *td, BOOL bIsOptional, short nParamFlag, int nAddPtr, BSTR varname, CString &s); // used by discoverInterface
	BOOL isOptionalParam(int i, short nbparams, short nboptionalparams);
};



class CServerContext : public CContext
{
	// Members
	//
protected:
	IDispatch*		m_pServerObj;

	// Construction
	//
public:
	CServerContext();
	virtual ~CServerContext();
	
	// Methods
	//
public:
	virtual void init();
	virtual void apply(/*in*/CStringArray &ar); // override this method


	IDispatch *getInstance();

};









#endif // __CCONTEXT__