
#include "stdafx.h"
#include "HostSite.h"
#include "Context.h"
#include "XmlIdDef.h"
#include "XmlFileManager.h"
#include "xmlcharsetconversion.h"
#include "xmlreader.h"



HRESULT gLastError; // global var



CHostSite::CHostSite()
{
	CoInitialize(NULL);

	XmlhashTableInit(&m_contexts);

}

CHostSite::~CHostSite()
{
	CoUninitialize();

	XmlhashTableDestroy(&m_contexts);
}





// ------------------------- xml implementation --------------------------


BOOL CHostSite::parseXmlFile(char *szFilename)
{
	BOOL bError = FALSE;

	if (szFilename==NULL || strlen(szFilename)==0)
	{
		trace2(TRC_ERROR,"CHostSite::parseXmlFile",SAXML_ERRORSTRING_BADURL);
		return FALSE;
	}

	CContext *pNullContext = new CContext();
	if (!pNullContext)
		return FALSE;
	pNullContext->setHostSite(this);

	CContext *pCurContext = pNullContext;

	// read and parse the xml file
	//

	XmlReader r;
	BOOL bReturn = TRUE;

	r.ShowMsgBoxOnError();
	r.Open( CString(szFilename) );

	CStringArray ar; // array of attributes
	CStringArray stackelements; // stack of nested elements
	CString szLastElement;

	while ( r.ReadString() && bReturn )
	{
		switch ( r.GetNodeType() )
		{
			case NODETYPE_BEGINELEMENT:
			{
				CString szElement;
				r.GetNodeName(szElement);
				szElement = GetUnescapedXmlString(szElement);

				szLastElement.Empty();
				if (stackelements.GetSize() > 0)
					szLastElement = stackelements.GetAt( stackelements.GetSize() - 1 );

				if (pCurContext && (szElement.CompareNoCase(XMLPROPERTY) || 
					szElement.CompareNoCase(XMLMETHOD)) && ar.GetSize()>0 )
				{
					// get context
					CContext *pContext = getContext( szLastElement, pCurContext );
					if (pContext)
					{
						// pass attribs
						pContext->apply( ar );
						pCurContext = pContext;
					}
					ar.RemoveAll();

				}
				else // normal interface traversal
				{
					if ( szElement.GetLength() > 0 )
					{
						if (szLastElement.GetLength()>0 && ar.GetSize()>0 && pCurContext)
						{
							// create new context
							CContext *pContext = getContext( szLastElement, pCurContext );
							if (pContext)
							{
								// pass attribs
								pContext->apply( ar );
								pCurContext = pContext;
							}

						}
						ar.RemoveAll();

						stackelements.Add( szElement );
					}
				} // end normal interface traversal
			} 
			break;
			case NODETYPE_ENDELEMENT:
			{
				CString szElement;
				r.GetNodeName(szElement);
				szElement = GetUnescapedXmlString(szElement);

				szLastElement.Empty();
				if (stackelements.GetSize() > 0)
					szLastElement = stackelements.GetAt( stackelements.GetSize() - 1 );

				if (pCurContext && (szElement.CompareNoCase(XMLPROPERTY) || 
					szElement.CompareNoCase(XMLMETHOD)) && ar.GetSize()>0 )
				{
					// get context
					CContext *pContext = getContext( szLastElement, pCurContext );
					if (pContext)
					{
						// pass attribs
						pContext->apply( ar );
					}
					ar.RemoveAll();
				}
				else // normal interface traversal
				{
					if (szElement.GetLength()>0)
					{
						if (szLastElement.GetLength()>0 && ar.GetSize()>0)
						{
							// create new context
							CContext *pContext = getContext( szLastElement, pCurContext );
							if (pContext)
							{
								// pass attribs
								pContext->apply( ar );
								pCurContext = pContext;
							}

						}
						ar.RemoveAll();

		
						if (stackelements.GetSize() > 0)
							stackelements.RemoveAt ( stackelements.GetSize() - 1 );

						// destroy context
						if (pCurContext)
						{
							// next context = parent context
							pCurContext = pCurContext->getParent();

							// destroy context now, since we won't need it anymore
							closeContext( szElement );
						}

					}
				} // end normal interface traversal
			} 
			break;
			case NODETYPE_ATTRIB:
			{
				CString szName, szValue;
				r.GetAttrib(szName, szValue);
				szName = GetUnescapedXmlString(szName);
				szValue = GetUnescapedXmlString(szValue);

				CString szElement;
				r.GetNodeName(szElement);
				szElement = GetUnescapedXmlString(szElement);

				if (szElement.GetLength()>0)
				{
					ar.Add( szName );
					ar.Add( szValue );
				}

			} 
			break;

			// other nodetypes : we don't care

		}
	} // end read lines one by one

	r.Close();


	//
	// xml parsing finished

	removeContexts();

	delete pNullContext;

	return bError;
}







CContext *CHostSite::getContext(const LPTSTR szContext, CContext *pParentContext)
{
	if (!szContext || strlen(szContext)==0)
		return NULL;

	CContext *pContext=NULL;

	// Find an element in the table using hash keys : direct access almost granted
	XMLNAMED *pBlock = (XMLNAMED*) XmlhashTableLookup(&m_contexts, szContext, NULL);
	if (pBlock)
	{
		pContext = (CContext*) pBlock->ptr;
		return pContext;
	}
	
	// create new context
	pContext = createContext(szContext);
	if (pContext)
	{
		pContext->setHostSite( this );
		pContext->setParent( pParentContext );
		pContext->setName( szContext );
		pBlock = (XMLNAMED*) XmlhashTableLookup(&m_contexts, szContext, pContext);
		if (!pBlock) // hard crash error
		{
			CString s;
			s.Format("couldn't add %s context in internal process while parsing XML content\n", szContext);
			OutputDebugString(s);
		}
	}
	return pContext;
}


CContext *CHostSite::createContext(const LPTSTR szContext)
{
	if (_tcscmp(szContext, ISERVERCTX)==0)
		return new CServerContext();
	else
		return new CContext();
}


void CHostSite::closeContext(const LPTSTR szContext)
{
	if (!szContext || strlen(szContext)==0)
		return;

	CContext *pContext=NULL;

	// Find an element in the table using hash keys : direct access almost granted
	XMLNAMED *pBlock = (XMLNAMED*) XmlhashTableLookup(&m_contexts, szContext, NULL);
	if (pBlock)
	{
		pContext = (CContext*) pBlock->ptr;
		pBlock->ptr = NULL; // remove ptr reference to avoid double deletion (GPF!!)
	}

	delete pContext;
}

void CHostSite::removeContexts()
{
	XMLHASH_TABLE_ITER iter;
	XMLNAMED *pair = XmlhashTableFirstItem(&iter, &m_contexts);
	if (!pair)
		return;

	delete (CContext*) pair->ptr;

	while ( (pair=XmlhashTableNextItem(&iter, &m_contexts))!=NULL )
	{
		delete (CContext*) pair->ptr;
	}
}











///////////////////////////IDISPATCH INVOKE HELPER METHODS////////////////////////////////////////////////



HRESULT CDoItInvokeHelper::InvokeHelper_PropertyGet(IDispatch *pDisp, OLECHAR FAR* szProperty, VARIANT FAR &VarResult)
{
	HRESULT hr;
	DISPID dispid;
	DISPPARAMS dispparamsNoArgs = {NULL, NULL, 0, 0};

	if (!pDisp)
		return E_NOINTERFACE;

	if (!szProperty)
		return E_INVALIDARG;


	hr = pDisp->GetIDsOfNames( IID_NULL,    
								&szProperty,
								1, 
								LOCALE_USER_DEFAULT,    
								&dispid);

	if (SUCCEEDED(hr))
	{
		hr = pDisp->Invoke(dispid,
							IID_NULL,
							LOCALE_USER_DEFAULT,
							DISPATCH_PROPERTYGET,
							&dispparamsNoArgs, 
							&VarResult, 
							NULL, 
							NULL);

			// VarResult has the result
			// (remember to do ::SysFreeString if variant is of type BSTR)
	}

	gLastError = hr;

	return hr;
}

HRESULT CDoItInvokeHelper::InvokeHelper_PropertyPutDISPPARAMS(IDispatch *pDisp, OLECHAR FAR* szProperty, DISPPARAMS &dispParams, VARIANT FAR &VarResult)
{
	HRESULT hr;
	DISPID dispid;

	if (!pDisp)
		return E_NOINTERFACE;

	if (!szProperty)
		return E_INVALIDARG;

	DISPID nameddispids = DISPID_PROPERTYPUT;

	dispParams.rgdispidNamedArgs = &nameddispids;
	dispParams.cNamedArgs = 1;





	hr = pDisp->GetIDsOfNames( IID_NULL,    
								&szProperty,
								1, 
								LOCALE_USER_DEFAULT,    
								&dispid);

	if (SUCCEEDED(hr))
	{

		hr = pDisp->Invoke(dispid,
							IID_NULL,
							LOCALE_USER_DEFAULT,
							DISPATCH_PROPERTYPUT,
							&dispParams, 
							&VarResult, 
							NULL, 
							NULL);

			// VarResult has the result
			// (remember to do ::SysFreeString if variant is of type BSTR)
	}

	gLastError = hr;

	return hr;
}

HRESULT CDoItInvokeHelper::InvokeHelper_Method_NoParam(IDispatch *pDisp, OLECHAR FAR* szMethod, VARIANT FAR &VarResult)
{
	HRESULT hr;
	DISPID dispid;

	if (!pDisp)
		return E_NOINTERFACE;

	if (!szMethod)
		return E_INVALIDARG;


	hr = pDisp->GetIDsOfNames( IID_NULL,    
								&szMethod,
								1, 
								LOCALE_USER_DEFAULT,    
								&dispid);

	if (SUCCEEDED(hr))
	{

		DISPPARAMS dispparamsNoArgs = {NULL, NULL, 0, 0};

		hr = pDisp->Invoke(dispid,
							IID_NULL,
							LOCALE_USER_DEFAULT,
							DISPATCH_METHOD,
							&dispparamsNoArgs, 
							&VarResult, 
							NULL, 
							NULL);

			// VarResult has the result
			// (remember to do ::SysFreeString if variant is of type BSTR)
	}

	gLastError = hr;

	return hr;
}


HRESULT CDoItInvokeHelper::InvokeHelper_Method_ParamLong(IDispatch *pDisp, OLECHAR FAR* szMethod, long nParamValue, VARIANT FAR &VarResult)
{
	HRESULT hr;
	DISPID dispid;

	if (!pDisp)
		return E_NOINTERFACE;

	if (!szMethod)
		return E_INVALIDARG;


	hr = pDisp->GetIDsOfNames( IID_NULL,    
								&szMethod,
								1, 
								LOCALE_USER_DEFAULT,    
								&dispid);

	if (SUCCEEDED(hr))
	{

		VARIANT v1;
		v1.vt = VT_I4;
		v1.lVal = nParamValue;
		DISPPARAMS dispparams;
		dispparams.rgvarg = &v1;
		dispparams.cArgs = 1;
		dispparams.cNamedArgs = 0;


		hr = pDisp->Invoke(dispid,
							IID_NULL,
							LOCALE_USER_DEFAULT,
							DISPATCH_METHOD,
							&dispparams, 
							&VarResult, 
							NULL, 
							NULL);

			// VarResult has the result
			// (remember to do ::SysFreeString if variant is of type BSTR)
	}

	gLastError = hr;

	return hr;
}

HRESULT CDoItInvokeHelper::InvokeHelper_Method_ParamBSTR(IDispatch *pDisp, OLECHAR FAR* szMethod, BSTR &szParamValue, VARIANT FAR &VarResult)
{
	HRESULT hr;
	DISPID dispid;

	if (!pDisp)
		return E_NOINTERFACE;

	if (!szMethod)
		return E_INVALIDARG;


	hr = pDisp->GetIDsOfNames( IID_NULL,    
								&szMethod,
								1, 
								LOCALE_USER_DEFAULT,    
								&dispid);

	if (SUCCEEDED(hr))
	{

		VARIANT v1;
		v1.vt = VT_BSTR;
		v1.bstrVal = szParamValue;
		DISPPARAMS dispparams;
		dispparams.rgvarg = &v1;
		dispparams.cArgs = 1;
		dispparams.cNamedArgs = 0;

		hr = pDisp->Invoke(dispid,
							IID_NULL,
							LOCALE_USER_DEFAULT,
							DISPATCH_METHOD,
							&dispparams, 
							&VarResult, 
							NULL, 
							NULL);

			// VarResult has the result
			// (remember to do ::SysFreeString if variant is of type BSTR)
	}

	gLastError = hr;

	return hr;
}

HRESULT CDoItInvokeHelper::InvokeHelper_Method_ParamDISPPARAMS(IDispatch *pDisp, OLECHAR FAR* szMethod, DISPPARAMS &dispParams, VARIANT FAR &VarResult)
{
	HRESULT hr;
	DISPID dispid;

	if (!pDisp)
		return E_NOINTERFACE;

	if (!szMethod)
		return E_INVALIDARG;


	hr = pDisp->GetIDsOfNames( IID_NULL,    
								&szMethod,
								1, 
								LOCALE_USER_DEFAULT,    
								&dispid);

	if (SUCCEEDED(hr))
	{

		hr = pDisp->Invoke(dispid,
							IID_NULL,
							LOCALE_USER_DEFAULT,
							DISPATCH_METHOD,
							&dispParams, 
							&VarResult, 
							NULL, 
							NULL);

			// VarResult has the result
			// (remember to do ::SysFreeString if variant is of type BSTR)
	}

	gLastError = hr;

	return hr;
}

















///////////////////////////HASH CODE TABLE ////////////////////////////////////////////////


#define XMLINIT_SIZE 64

static int Xmlkeyeq(const char *s1, const char *s2)
{
  for (; *s1 == *s2; s1++, s2++)
    if (*s1 == 0)
      return 1;
  return 0;
}

static unsigned long Xmlhash(const char *s)
{
  unsigned long h = 0;
  while (*s)
    h = (h << 5) + h + (unsigned char)*s++;
  return h;
}

XMLNAMED *XmlhashTableLookup(XMLHASH_TABLE *table, const char *name, const void *ptr)
{
  size_t i;
  if (table->size == 0) // non initialized table
  {
    table->v = (XMLNAMED**) calloc(XMLINIT_SIZE, sizeof(XMLNAMED *));
    if (!table->v)
      return NULL;
    table->size = XMLINIT_SIZE;
    table->usedLim = XMLINIT_SIZE / 2;
    i = Xmlhash(name) & (table->size - 1);
  }
  else // find name in the table
  {
    unsigned long h = Xmlhash(name);
    for (i = h & (table->size - 1); table->v[i]; i == 0 ? i = table->size - 1 : --i) 
	{
      if (Xmlkeyeq(name, table->v[i]->name))
	  {
		  if( ! table->v[i]->obsolete)
			return table->v[i]; // name found
	  }
    }

	// not found, prepare addition to the table
    if (table->used == table->usedLim) 
	{
      // check for overflow
      size_t newSize = table->size * 2;
      XMLNAMED **newV = (XMLNAMED**) calloc(newSize, sizeof(XMLNAMED *));
      if (!newV)
		return NULL;
      for (i = 0; i < table->size; i++)
		if (table->v[i]) 
		{
			size_t j;
			for (j = Xmlhash(table->v[i]->name) & (newSize - 1);
					newV[j];
					j == 0 ? j = newSize - 1 : --j)
				;

			newV[j] = table->v[i];
		}
      free(table->v);
      table->v = newV;
      table->size = newSize;
      table->usedLim = newSize/2;
      for (i = h & (table->size - 1);  table->v[i];  i == 0 ? i = table->size - 1 : --i)
			;
    }
  }

  if (!ptr) 
	  return NULL;

  // new item added to the table
  table->v[i] = (XMLNAMED*) calloc(1, sizeof(XMLNAMED) );
  if (!table->v[i])
      return NULL;
  strcpy(table->v[i]->name, name);
  table->v[i]->ptr = (void*)ptr;
  table->v[i]->obsolete = 0;
  (table->used)++;

  return table->v[i];
}

void XmlhashTableRemove(XMLHASH_TABLE *table, const char *name)
{
	XMLNAMED * pFound = XmlhashTableLookup(table, name, NULL);

	if( pFound)
	{
		pFound->obsolete = 1;
		if( table)
			table->used--;
	}
}

void XmlhashTableDestroy(XMLHASH_TABLE *table)
{
  for (size_t i = 0; i < table->size; i++) 
  {
    XMLNAMED *p = table->v[i];
    if (p)
      free(p);
  }

  if (table->v)
	free(table->v);

  // init again
  XmlhashTableInit(table);

}

void XmlhashTableInit(XMLHASH_TABLE *p)
{
  p->size = 0;
  p->usedLim = 0;
  p->used = 0;
  p->v = NULL;
}

size_t XmlhashTableSize(XMLHASH_TABLE *p)
{
	return p? p->used : 0;
}


XMLNAMED *XmlhashTableFirstItem(XMLHASH_TABLE_ITER *iter, XMLHASH_TABLE *p)
{
	if (!iter)
		return NULL;

	iter->p = p->v;
	iter->end = iter->p + p->size;

	return XmlhashTableNextItem(iter, p);
}

XMLNAMED *XmlhashTableNextItem(XMLHASH_TABLE_ITER *iter, XMLHASH_TABLE *p)
{
	while (iter->p != iter->end) 
	{
		XMLNAMED *tem = *(iter->p)++;
		if (tem && ( ! tem->obsolete))
			return tem;
	}
	return NULL;
}
