
#ifndef _XMLFILEMANAGER_
#define _XMLFILEMANAGER_


#include "XmlMsgInterface.h"
#ifdef _WIN32
	#include <wininet.h>
#endif

class CXmlFileManager : public IXmlMsgInterface
{

	// Members
	//
protected:
	char	m_szURL[SAXML_URLMAXLENGTH];
	int		m_cnt;
	BOOL	m_bSkipNow;
	BOOL	m_bPromptMode; // if TRUE, optimization msg box is shown
	void*	m_cpParent; // used to send message (SetParamInt, ...)

	static long m_nLastError;

	// Constructor
	//
public:
	CXmlFileManager();

	// Methods
	//
	static  CXmlFileManager *chooseResourceManager(char *szURL);
	void	setPromptMode(BOOL bPrompt=TRUE);
	void	peekMessage();
	void	skipNow(BOOL bSkipNow=TRUE);
	BOOL	shouldSkipNow();

	virtual BOOL SetParamInt(int nParamID, int nValue);
	virtual BOOL SetParamString(int nParamID, char *szValue);
	virtual BOOL SetParamPtr(int nParamID, void *pValue);

	virtual int  GetParamInt(int nParamID);
	virtual char *GetParamString(int nParamID);
	virtual void *GetParamPtr(int nParamID);
 
	virtual void SetMsgHandler(void *pHandler);

	virtual BOOL Open(char *szURL) = 0; // VIRTUAL PURE
	virtual BOOL Read(char *szInputBuffer, int nMaxLength, int &nActualLength) = 0; // VIRTUAL PURE
	virtual BOOL Close() = 0; // VIRTUAL PURE

	virtual void  SetParent(void *pParent);
	virtual void* GetParent();


	static void setLastError(long nError);
	static long getLastError();


	// Prohibited use
	//
private:
	CXmlFileManager& operator=(const CXmlFileManager &obj);

};


class CXmlLocalFileManager : public CXmlFileManager
{
	// Members
	//
protected:
	FILE *fp; // file specific variable

	// Constructor
	//
public:
	CXmlLocalFileManager();
	virtual ~CXmlLocalFileManager();

	// Methods
	//
	virtual BOOL Open(char *szURL); // the URL is actually a fully qualified local filepath
	virtual BOOL Read(char *szInputBuffer, int nMaxLength, int &nActualLength);
	virtual BOOL Close();


	// Prohibited use
	//
private:
	CXmlLocalFileManager& operator=(const CXmlLocalFileManager &obj);
};


class CXmlInternetFileManager : public CXmlFileManager
{
	// Members
	//
protected:
	char		m_szProxy[MAX_PATH];
	int			m_nProxyPort;
	HINTERNET	m_hInternetSession; // Wininet specific handle
	HINTERNET	m_hInternetFile;  // Wininet specific handle


	// Constructor
	//
public:
	CXmlInternetFileManager();
	virtual ~CXmlInternetFileManager();

	// Methods
	//
	virtual BOOL Open(char *szURL);
	virtual BOOL Read(char *szInputBuffer, int nMaxLength, int &nActualLength);
	virtual BOOL Close();

	// Prohibited use
	//
private:
	CXmlInternetFileManager& operator=(const CXmlInternetFileManager &obj);
};



class CXmlSecuredFileManager : public CXmlFileManager
{
	// Members
	//
protected:

	// Constructor
	//
public:
	CXmlSecuredFileManager();
	virtual ~CXmlSecuredFileManager();

	// Methods
	//
	virtual BOOL Open(char *szURL);
	virtual BOOL Read(char *szInputBuffer, int nMaxLength, int &nActualLength);
	virtual BOOL Close();

	// Prohibited use
	//
private:
	CXmlSecuredFileManager& operator=(const CXmlSecuredFileManager &obj);
};

#endif
