
#include "stdafx.h"
#include "xmlcharsetconversion.h"



CString GetEscapedXmlString(CString &szInput)
{
	CString s;
	if (szInput.GetLength()==0) return s;

	long nLength = szInput.GetLength();
	for (long i=0; i<nLength; i++)
	{
		char c = szInput.GetAt(i);
		if (c=='&')
			s += "&amp;";
		else if (c=='"')
			s += "&quot;";
		else if (c=='<')
			s += "&lt;";
		else
			s += c;
	}

	return UTF8Conversion(s);
}


CString UTF8Conversion(CString &s)
{

	wchar_t *pWideString = new wchar_t[s.GetLength()+1];
	if (!pWideString) return s;

	// local charset to widechar
	//

	int nActualWideLength = ::MultiByteToWideChar(CP_ACP,
			0,
			s.GetBuffer(0),
			s.GetLength(),
			pWideString,
			s.GetLength());

	pWideString[nActualWideLength] = 0;

	// widechar to UTF8
	//		
	char *temps = new char[2*nActualWideLength+5]; // 2*x+5 (why not?)
	if (!temps)
	{
		delete [] pWideString;
		return s;
	}

	int nActualLength = ::WideCharToMultiByte(CP_UTF8,
			0,
			pWideString,
			-1,
			temps,
			2*nActualWideLength+5,
			NULL,
			NULL);

	CString st = temps;

	delete [] temps;

	delete [] pWideString;

	return st;

}


CString GetUnescapedXmlString(CString &szInput) // Xml escape chars (&quot; ==> ", &lt; ==> <, &amp; ==> &);
{
	CString s;
	if (szInput.GetLength()==0) return s;

	s = FromUTF8Conversion(szInput);

	int nLt;
	while ( (nLt=s.Find("&lt;",0))>-1 )
		s = s.Left(nLt) + "<" + s.Right( s.GetLength()-(nLt+strlen("&lt;")) );

	int nQuot;
	while ( (nQuot=s.Find("&quot;",0))>-1 )
		s = s.Left(nQuot) + "\"" + s.Right( s.GetLength()-(nQuot+strlen("&quot;")) );

	int nAmp;
	while ( (nAmp=s.Find("&amp;",0))>-1 )
		s = s.Left(nAmp) + "&" + s.Right( s.GetLength()-(nAmp+strlen("&amp;")) );

	return s;
}

CString FromUTF8Conversion(CString &s) // UTF8 ==> local charset
{
	wchar_t *pWideString = new wchar_t[s.GetLength()+1];
	if (!pWideString) return s;

	// UTF8 to widechar
	//

	int nActualWideLength = ::MultiByteToWideChar(CP_UTF8,
			0,
			s.GetBuffer(0),
			s.GetLength(),
			pWideString,
			s.GetLength());

	pWideString[nActualWideLength] = 0;

	// widechar to local charset
	//		
	char *temps = new char[nActualWideLength+1];
	if (!temps)
	{
		delete [] pWideString;
		return s;
	}

	int nActualLength = ::WideCharToMultiByte(CP_ACP,
			0,
			pWideString,
			-1,
			temps,
			nActualWideLength+1,
			NULL,
			NULL);

	CString st = temps;

	delete [] temps;

	delete [] pWideString;

	return st;
}










void BSTR_to_CString(/*in*/BSTR bstr, /*out*/CString &s)
{
	s.Empty();

	_bstr_t bstrTag(bstr);

	UINT uLen = bstrTag.length();
	if (uLen==0) return;

	TCHAR *szTemp= new TCHAR[uLen+1];
	memset (szTemp, 0, uLen+1);

	LPOLESTR wszTag = bstrTag; // get ptr to Unicode

	int iBytes = ::WideCharToMultiByte(CP_ACP, 0,
			wszTag, uLen,
			szTemp, uLen, NULL, NULL);

	s = szTemp;

	delete [] szTemp;

	::SysFreeString(bstr);
}


void VARIANT_to_CString(/*in*/VARIANT &vt, /*out*/CString &s)
{
	s.Empty();

	if (vt.vt!=VT_BSTR) return;

	_bstr_t bstrTag(vt);

	UINT uLen = bstrTag.length();
	if (uLen==0) return;

	TCHAR *szTemp= new TCHAR[uLen+1];
	memset (szTemp, 0, uLen+1);

	LPOLESTR wszTag = bstrTag; // get ptr to Unicode

	int iBytes = ::WideCharToMultiByte(CP_ACP, 0,
			wszTag, uLen,
			szTemp, uLen, NULL, NULL);

	s = szTemp;

	delete [] szTemp;

	::SysFreeString(vt.bstrVal);
}


void TokenizeString(/*in*/CString &szInput, char cToken, /*out*/ArrayCString &arrStrings)
{
	long i;

	CString s = szInput;

	arrStrings.RemoveAll();

	while ( (i=s.Find(cToken,0))>-1 )
	{
		arrStrings.Add( s.Left(i) );
		s = s.Right( s.GetLength()-(i+1) );
	}
	if (!s.IsEmpty())
		arrStrings.Add ( s );
}


void TokenizeString(/*in*/CString &szInput, /*in*/CString &szTokenString, /*out*/ArrayCString &arrStrings)
{
	long i;

	CString s = szInput;

	arrStrings.RemoveAll();

	while ( (i=s.Find(szTokenString,0))>-1 )
	{
		arrStrings.Add( s.Left(i) );
		s = s.Right( s.GetLength()-(i+szTokenString.GetLength()) );
	}
	if (!s.IsEmpty())
		arrStrings.Add ( s );
}

void UntokenizeString(/*in*/ArrayCString &arrStrings, char cToken, /*out*/CString &szOutput)
{
	szOutput.Empty();

	for (long i=0;i<arrStrings.GetSize(); i++)
	{
		if (!szOutput.IsEmpty()) szOutput += cToken;

		szOutput += arrStrings.GetAt(i);
	}
}



void __stdcall _com_issue_error(HRESULT)
{}
